﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Threading.Tasks;
using System.Windows.Forms;
using HeuristicLab.MainForm;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Problems.Instances.DataAnalysis.Regression.Matlab;
using HeuristicLab.Problems.Instances.DataAnalysis.Views.Regression.Matlab;

namespace HeuristicLab.Problems.Instances.DataAnalysis.Views {
  [View("Regression InstanceProvider View")]
  [Content(typeof(RegressionInstanceProvider), IsDefaultView = true)]
  public partial class RegressionInstanceProviderView : DataAnalysisInstanceProviderView<IRegressionProblemData> {

    public new RegressionInstanceProvider Content {
      get { return (RegressionInstanceProvider)base.Content; }
      set { base.Content = value; }
    }

    public RegressionInstanceProviderView() {
      InitializeComponent();
    }

    protected override void importButton_Click(object sender, EventArgs e) {
      if (Content is RegressionMatlabInstanceProvider) {
        ImportMatlabData();
      } else {
        ImportCSVData();
      }
    }

    private void ImportCSVData() {
      var importTypeDialog = new RegressionImportDialog();
      if (importTypeDialog.ShowDialog() == DialogResult.OK) {
        IRegressionProblemData instance = null;

        Task.Factory.StartNew(() => {
          var mainForm = (MainForm.WindowsForms.MainForm)MainFormManager.MainForm;
          // lock active view and show progress bar
          IContentView activeView = (IContentView)MainFormManager.MainForm.ActiveView;

          try {
            var progress = mainForm.AddOperationProgressToContent(activeView.Content,
              "Loading problem instance.");

            Content.ProgressChanged +=
              (o, args) => { progress.ProgressValue = args.ProgressPercentage / 100.0; };

            instance = Content.ImportData(importTypeDialog.Path, importTypeDialog.ImportType,
              importTypeDialog.CSVFormat);
          } catch (Exception ex) {
            ErrorWhileParsing(ex);
            return;
          } finally {
            mainForm.RemoveOperationProgressFromContent(activeView.Content);
          }

          try {
            GenericConsumer.Load(instance);
          } catch (Exception ex) {
            ErrorWhileLoading(ex, importTypeDialog.Path);
          } finally {
            Invoke((Action)(() => instancesComboBox.SelectedIndex = -1));
          }
        });
      }
    }


    private void ImportMatlabData() {
      var importTypeDialog = new RegressionMatlabImportDialog();
      if (importTypeDialog.ShowDialog() == DialogResult.OK) {
        IRegressionProblemData instance = null;

        Task.Factory.StartNew(() => {
          var mainForm = (MainForm.WindowsForms.MainForm)MainFormManager.MainForm;
          // lock active view and show progress bar
          IContentView activeView = (IContentView)MainFormManager.MainForm.ActiveView;

          try {
            var progress = mainForm.AddOperationProgressToContent(activeView.Content,
              "Loading problem instance.");

            Content.ProgressChanged +=
              (o, args) => { progress.ProgressValue = args.ProgressPercentage / 100.0; };

            instance = (Content as RegressionMatlabInstanceProvider).ImportData(importTypeDialog.Path, importTypeDialog.ImportType, importTypeDialog.SelectedVariables);
          } catch (Exception ex) {
            ErrorWhileParsing(ex);
            return;
          } finally {
            mainForm.RemoveOperationProgressFromContent(activeView.Content);
          }

          try {
            GenericConsumer.Load(instance);
          } catch (Exception ex) {
            ErrorWhileLoading(ex, importTypeDialog.Path);
          } finally {
            Invoke((Action)(() => instancesComboBox.SelectedIndex = -1));
          }
        });
      }
    }    
  }
}
